"""
Serializer code for cobbler

Copyright 2006, Red Hat, Inc
Michael DeHaan <mdehaan@redhat.com>

This software may be freely redistributed under the terms of the GNU
general public license.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""

import distutils.sysconfig
import os
import sys
import glob

plib = distutils.sysconfig.get_python_lib()
mod_path="%s/cobbler" % plib
sys.path.insert(0, mod_path)

from rhpl.translate import _, N_, textdomain, utf8
import utils
import yaml   # Howell-Clark version
import cexceptions
import os

def register():
    """
    The mandatory cobbler module registration hook.
    """
    return "serializer"

def serialize(obj):
    """
    Basic serialization means rewriting the listings file
    and /all/ data files, which makes it rather costly.  It
    is only to be run on upgrades.
    """
    __update_listings_file(obj)
    __rewrite_contents_files(obj)
    return True

def serialize_item(obj, item):
    """
    Item serialization means writing just the one file, though
    we must also rewrite the listings as currently implemented.
    """
    __update_listings_file(obj)
    __rewrite_contents_file(obj,item)
    return True

def serialize_delete(obj, item):
    """
    A deletion involves updating listings as well as removing
    the data, though technically we don't have to remove the
    data except to save space.
    """
    __update_listings_file(obj)
    filename = __get_filename(obj.collection_type(), item)
    if os.path.exists(filename):
        os.remove(filename)
    return True

def __open_for_write(filename):
    """
    Helper function to create intermediate directories
    for config files.
    """
    try:
        fd = open(filename,"w+")
    except IOError, ioe:
        dirname = os.path.dirname(filename)
        try:
           os.makedirs(dirname)
        except OSError, ose:
           pass
        try:
           fd = open(filename,"w+")
        except IOError, ioe3:
           raise cexceptions.CX(_("Need permissions to write to %s") % filename)
    return fd
     

def __data_to_yaml(filename,datastruct):
    """
    Serializate datastruct to filename
    """
    fd = __open_for_write(filename)
    encoded = yaml.dump(datastruct)
    fd.write(encoded)
    fd.close()

def __rewrite_contents_files(collection):
    """
    For all items in collection, save to disk.
    """
    for item in collection:
        __rewrite_contents_file(collection,item)
    return True

def __rewrite_contents_file(collection,item):
    """
    Save a specific item to disk.
    """
    filename = __get_filename(collection.collection_type(), item_name=item.name)
    return __data_to_yaml(filename, item.to_datastruct())

def __update_listings_file(obj):
    """
    Updates the ... listing file.
    """
    filename = __get_filename(obj.collection_type())
    datastruct = []
    for x in obj:
        datastruct.append(x.name)
    return __data_to_yaml(filename, datastruct)    

def __load_from_yaml(filename):
    """ 
    Given a yaml file, filename, return datastructure contents.
    """
    try:
        fd = open(filename,"r")
    except IOError, ioe:
        return [{}]
    data = fd.read()
    datastruct = yaml.load(data).next()  # first record
    fd.close()
    # print "DEBUG: loaded from yaml: %s" % datastruct
    return datastruct

def __get_filename(collection_type,item_name=None):
    """ 
    Get the filename for a listing file, or for an object.
    """
    # FIXME: use this everywhere
    ending = collection_type
    if not ending.endswith("s"):
        ending = ending + "s"
    if item_name is None:
       return "/var/lib/cobbler/config/index/%s" % ending
    else:
       return "/var/lib/cobbler/config/data/%s/%s" % (ending, item_name)

def deserialize_raw(collection_type):
    """
    Load the raw data for the collection.
    FIXME: though the overall performance of fastyaml is faster,
    this is more expensive.  Tradeoffs!
    """
    return __deserialize_lowlevel(collection_type,topological=False)

def deserialize_item_raw(collection_type,item_name):
    """
    Load data for a specific item without creating an object for it.
    """ 
    filename = __get_filename(collection_type,item_name=item_name)
    return __load_from_yaml(filename)

def deserialize(obj,topological=False):
    datastruct =  __deserialize_lowlevel(obj.collection_type(),topological)
    return obj.from_datastruct(datastruct)

def __deserialize_lowlevel(collection_type,topological=False):
    """
    Populate an existing object with the contents of datastruct.
    Object must "implement" Serializable.  Returns True assuming
    files could be read and contained decent YAML.  Otherwise returns
    False.
    """
    filename = __get_filename(collection_type)
    if not os.path.exists(filename):
       return obj.from_datastruct([])
    datastruct = __load_from_yaml(filename)

    datastruct2 = []
    for name in datastruct:
        filename2 = __get_filename(collection_type, name)
        datastruct2.append(__load_from_yaml(filename2))
    datastruct = datastruct2

    if topological and type(datastruct) == list:
       # in order to build the graph links from the flat list, sort by the
       # depth of items in the graph.  If an object doesn't have a depth, sort it as
       # if the depth were 0.  It will be assigned a proper depth at serialization
       # time.  This is a bit cleaner implementation wise than a topological sort,
       # though that would make a shiny upgrade.
       datastruct.sort(__depth_cmp)
    return datastruct

def __depth_cmp(item1, item2):
    if not item1.has_key("depth"):
       return 1
    if not item2.has_key("depth"):
       return -1
    return cmp(item1["depth"],item2["depth"])

